@@ -17,6 +17,7 @@ from syned.beamline.beamline_element import BeamlineElement
 from syned.beamline.shape import *
 
 from wofry.propagator.propagator import PropagationManager, PropagationElements, PropagationParameters
+from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D
 from wofryimpl.propagator.propagators2D.fresnel import Fresnel2D
 from wofryimpl.propagator.propagators2D.fresnel_convolution import FresnelConvolution2D
 from wofryimpl.propagator.propagators2D.fraunhofer import Fraunhofer2D
@@ -27,6 +28,7 @@ from orangecontrib.wofry.util.wofry_objects import WofryData
 from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget
 
 
+
 def initialize_default_propagator_2D():
     propagator = PropagationManager.Instance()
 
@@ -58,6 +60,7 @@ class OWWOOpticalElement(WofryWidget, WidgetDecorator):
                 "id":"Trigger"}]
 
     inputs = [("WofryData", WofryData, "set_input"),
+              ("GenericWavefront2D", GenericWavefront2D, "set_input"),
               WidgetDecorator.syned_input_data()[0]]
 
     oe_name         = Setting("")
@@ -321,8 +324,10 @@ class OWWOOpticalElement(WofryWidget, WidgetDecorator):
         if not wofry_data is None:
             if isinstance(wofry_data, WofryData):
                 self.input_data = wofry_data
+            elif isinstance(wofry_data, GenericWavefront2D):
+                self.input_data = WofryData(wavefront=wofry_data)
             else:
-                raise Exception("Only wofry_data allowed as input")
+                raise Exception("Bad input.")
 
             if self.is_automatic_execution:
                 self.propagate_wavefront()
@@ -392,7 +397,15 @@ class OWWOOpticalElement(WofryWidget, WidgetDecorator):
                 raise Exception("Syned Data not correct: Empty Beamline Element")
 
     def receive_specific_syned_data(self, optical_element):
-        raise NotImplementedError()
+        if not optical_element is None:
+            if isinstance(optical_element, Lens):
+                self.lens_radius = optical_element._radius
+                self.wall_thickness = optical_element._thickness
+                self.material = optical_element._material
+            else:
+                raise Exception("Syned Data not correct: Optical Element is not a Lens")
+        else:
+            raise Exception("Syned Data not correct: Empty Optical Element")
 
     def callResetSettings(self):
         if ConfirmDialog.confirmed(parent=self, message="Confirm Reset of the Fields?"):
